@@ -7,7 +7,7 @@
  * will only be given back to netlist when refcount is zero.
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000, 2001 VideoLAN
- * $Id: dvd_netlist.c,v 1.5 2001/04/06 09:15:47 sam Exp $
+ * $Id: dvd_netlist.c,v 1.9 2001/06/03 12:47:21 sam Exp $
  *
  * Authors: Henri Fallon <henri@videolan.org>
  *          Stphane Borel <stef@videolan.org>
@@ -35,8 +35,17 @@
 #include <stdlib.h>
 #include <string.h>                                    /* memcpy(), memset() */
 #include <sys/types.h>
-#include <sys/uio.h>                                         /* struct iovec */
-#include <unistd.h>
+
+#ifdef HAVE_UNISTD_H
+#   include <unistd.h>
+#endif
+
+#if defined( WIN32 )
+#   include <io.h>
+#   include "input_iovec.h"
+#else
+#   include <sys/uio.h>                                      /* struct iovec */
+#endif
 
 #include "config.h"
 #include "common.h"
@@ -51,6 +60,9 @@
 #include "input.h"
 #include "dvd_netlist.h"
 
+#include "modules.h"
+#include "modules_export.h"
+
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
