@@ -2,7 +2,7 @@
  * idctmmxext.c : MMX EXT IDCT module
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: idctmmxext.c,v 1.6 2001/04/15 04:19:57 sam Exp $
+ * $Id: idctmmxext.c,v 1.10 2001/06/03 12:47:21 sam Exp $
  *
  * Authors: Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
  *          Michel Lespinasse <walken@zoy.org>
@@ -48,11 +48,14 @@
 #include "modules.h"
 #include "modules_inner.h"
 
-#include "idct.h"
+#include "vdec_block.h"
+#include "vdec_idct.h"
 
 #include "attributes.h"
 #include "mmx.h"
 
+#include "modules_export.h"
+
 /*****************************************************************************
  * Local prototypes.
  *****************************************************************************/
@@ -60,71 +63,26 @@ static void idct_getfunctions( function_list_t * p_function_list );
 static int  idct_Probe      ( probedata_t *p_data );
 static void vdec_NormScan   ( u8 ppi_scan[2][64] );
 
-
 /*****************************************************************************
  * Build configuration tree.
  *****************************************************************************/
 MODULE_CONFIG_START
 ADD_WINDOW( "Configuration for MMX EXT IDCT module" )
     ADD_COMMENT( "Ha, ha -- nothing to configure yet" )
-MODULE_CONFIG_END
-
-/*****************************************************************************
- * InitModule: get the module structure and configuration.
- *****************************************************************************
- * We have to fill psz_name, psz_longname and psz_version. These variables
- * will be strdup()ed later by the main application because the module can
- * be unloaded later to save memory, and we want to be able to access this
- * data even after the module has been unloaded.
- *****************************************************************************/
-MODULE_INIT
-{
-    p_module->psz_name = MODULE_STRING;
-    p_module->psz_longname = "MMX EXT IDCT module";
-    p_module->psz_version = VERSION;
+MODULE_CONFIG_STOP
 
+MODULE_INIT_START
     p_module->i_capabilities = MODULE_CAPABILITY_NULL
                                 | MODULE_CAPABILITY_IDCT;
+    p_module->psz_longname = "MMX EXT IDCT module";
+MODULE_INIT_STOP
 
-    return( 0 );
-}
-
-/*****************************************************************************
- * ActivateModule: set the module to an usable state.
- *****************************************************************************
- * This function fills the capability functions and the configuration
- * structure. Once ActivateModule() has been called, the i_usage can
- * be set to 0 and calls to NeedModule() be made to increment it. To unload
- * the module, one has to wait until i_usage == 0 and call DeactivateModule().
- *****************************************************************************/
-MODULE_ACTIVATE
-{
-    p_module->p_functions = malloc( sizeof( module_functions_t ) );
-    if( p_module->p_functions == NULL )
-    {
-        return( -1 );
-    }
-
+MODULE_ACTIVATE_START
     idct_getfunctions( &p_module->p_functions->idct );
+MODULE_ACTIVATE_STOP
 
-    p_module->p_config = p_config;
-
-    return( 0 );
-}
-
-/*****************************************************************************
- * DeactivateModule: make sure the module can be unloaded.
- *****************************************************************************
- * This function must only be called when i_usage == 0. If it successfully
- * returns, i_usage can be set to -1 and the module unloaded. Be careful to
- * lock usage_lock during the whole process.
- *****************************************************************************/
-MODULE_DEACTIVATE
-{
-    free( p_module->p_functions );
-
-    return( 0 );
-}
+MODULE_DEACTIVATE_START
+MODULE_DEACTIVATE_STOP
 
 /* Following functions are local */
 
@@ -134,10 +92,15 @@ MODULE_DEACTIVATE
 static void idct_getfunctions( function_list_t * p_function_list )
 {
     p_function_list->pf_probe = idct_Probe;
-    p_function_list->functions.idct.pf_init = _M( vdec_InitIDCT );
-    p_function_list->functions.idct.pf_sparse_idct = _M( vdec_SparseIDCT );
-    p_function_list->functions.idct.pf_idct = _M( vdec_IDCT );
-    p_function_list->functions.idct.pf_norm_scan = vdec_NormScan;
+#define F p_function_list->functions.idct
+    F.pf_idct_init = _M( vdec_InitIDCT );
+    F.pf_sparse_idct = _M( vdec_SparseIDCT );
+    F.pf_idct = _M( vdec_IDCT );
+    F.pf_norm_scan = vdec_NormScan;
+    F.pf_decode_init = _M( vdec_InitDecode );
+    F.pf_decode_mb_c = _M( vdec_DecodeMacroblockC );
+    F.pf_decode_mb_bw = _M( vdec_DecodeMacroblockBW );
+#undef F
 }
 
 /*****************************************************************************
@@ -145,21 +108,18 @@ static void idct_getfunctions( function_list_t * p_function_list )
  *****************************************************************************/
 static int idct_Probe( probedata_t *p_data )
 {
-    if( TestCPU( CPU_CAPABILITY_MMXEXT ) )
-    {
-        if( TestMethod( IDCT_METHOD_VAR, "idctmmxext" ) )
-        {
-            return( 999 );
-        }
-        else
-        {
-            return( 200 );
-        }
-    }
-    else
+    if( !TestCPU( CPU_CAPABILITY_MMXEXT ) )
     {
         return( 0 );
     }
+
+    if( TestMethod( IDCT_METHOD_VAR, "idctmmxext" ) )
+    {
+        return( 999 );
+    }
+
+    return( 200 );
+
 }
 
 /*****************************************************************************
