@@ -2,7 +2,7 @@
  * input_ps.c: PS demux and packet management
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: input_ps.c,v 1.20 2001/04/28 03:36:25 sam Exp $
+ * $Id: input_ps.c,v 1.28 2001/06/03 12:47:21 sam Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Cyril Deguet <asmax@via.ecp.fr>
@@ -32,14 +32,21 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
+
 #ifdef STRNCASECMP_IN_STRINGS_H
 #   include <strings.h>
 #endif
-#include <errno.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h>
+
+#ifdef HAVE_UNISTD_H
+#   include <unistd.h>
+#elif defined( _MSC_VER ) && defined( _WIN32 )
+#   include <io.h>
+#endif
+
 #include <fcntl.h>
 
 #include "config.h"
@@ -64,6 +71,7 @@
 #include "debug.h"
 
 #include "modules.h"
+#include "modules_export.h"
 
 /*****************************************************************************
  * Local prototypes
@@ -88,8 +96,8 @@ void _M( input_getfunctions )( function_list_t * p_function_list )
 #define input p_function_list->functions.input
     p_function_list->pf_probe = PSProbe;
     input.pf_init             = PSInit;
-    input.pf_open             = input_FileOpen;
-    input.pf_close            = input_FileClose;
+    input.pf_open             = NULL; /* Set in PSInit */
+    input.pf_close            = NULL;
     input.pf_end              = PSEnd;
     input.pf_set_area         = NULL;
     input.pf_read             = PSRead;
@@ -146,10 +154,10 @@ static int PSProbe( probedata_t *p_data )
 static void PSInit( input_thread_t * p_input )
 {
     thread_ps_data_t *  p_method;
-    packet_cache_t * 	p_packet_cache;
+    packet_cache_t *    p_packet_cache;
 
-    if( (p_method =
-         (thread_ps_data_t *)malloc( sizeof(thread_ps_data_t) )) == NULL )
+    p_method = (thread_ps_data_t *)malloc( sizeof(thread_ps_data_t) );
+    if( p_method == NULL )
     {
         intf_ErrMsg( "Out of memory" );
         p_input->b_error = 1;
@@ -166,6 +174,13 @@ static void PSInit( input_thread_t * p_input )
         return;
     }
     p_input->p_method_data = (void *)p_packet_cache;
+
+    /* Set callback */
+    p_input->pf_open  = p_input->pf_file_open;
+    p_input->pf_close = p_input->pf_file_close;
+
+    /* Initialize packet cache mutex */
+    vlc_mutex_init( &p_packet_cache->lock );
     
     /* allocates the data cache */
     p_packet_cache->data.p_stack = malloc( DATA_CACHE_SIZE * 
@@ -173,8 +188,8 @@ static void PSInit( input_thread_t * p_input )
     if ( p_packet_cache->data.p_stack == NULL )
     {
         intf_ErrMsg( "Out of memory" );
-    	p_input->b_error = 1;
-    	return;
+        p_input->b_error = 1;
+        return;
     }
     p_packet_cache->data.l_index = 0;
     
@@ -190,29 +205,31 @@ static void PSInit( input_thread_t * p_input )
     p_packet_cache->pes.l_index = 0;
     
     /* allocates the small buffer cache */
-    p_packet_cache->small.p_stack = malloc( SMALL_CACHE_SIZE * 
+    p_packet_cache->smallbuffer.p_stack = malloc( SMALL_CACHE_SIZE * 
         sizeof(packet_buffer_t) );
-    if ( p_packet_cache->small.p_stack == NULL )
+    if ( p_packet_cache->smallbuffer.p_stack == NULL )
     {
         intf_ErrMsg( "Out of memory" );
         p_input->b_error = 1;
         return;
     }
-    p_packet_cache->small.l_index = 0;
+    p_packet_cache->smallbuffer.l_index = 0;
     
     /* allocates the large buffer cache */
-    p_packet_cache->large.p_stack = malloc( LARGE_CACHE_SIZE * 
+    p_packet_cache->largebuffer.p_stack = malloc( LARGE_CACHE_SIZE * 
         sizeof(packet_buffer_t) );
-    if ( p_packet_cache->large.p_stack == NULL )
+    if ( p_packet_cache->largebuffer.p_stack == NULL )
     {
         intf_ErrMsg( "Out of memory" );
         p_input->b_error = 1;
         return;
     }
-    p_packet_cache->large.l_index = 0;
+    p_packet_cache->largebuffer.l_index = 0;
     
     /* Re-open the socket as a buffered FILE stream */
-    if( (p_method->stream = fdopen( p_input->i_handle, "r" )) == NULL )
+    p_method->stream = fdopen( p_input->i_handle, "r" );
+
+    if( p_method->stream == NULL )
     {
         intf_ErrMsg( "Cannot open file (%s)", strerror(errno) );
         p_input->b_error = 1;
@@ -335,7 +352,16 @@ static void PSInit( input_thread_t * p_input )
                         break;
 
                     case LPCM_AUDIO_ES:
-                        /* FIXME ! */
+                        if( main_GetIntVariable( INPUT_CHANNEL_VAR, 0 )
+                                == ((p_es->i_id & 0x1F00) >> 8) )
+                        switch( main_GetIntVariable( INPUT_AUDIO_VAR, 0 ) )
+                        {
+                        case 0:
+                            main_PutIntVariable( INPUT_AUDIO_VAR,
+                                                 REQUESTED_LPCM );
+                        case REQUESTED_LPCM:
+                            input_SelectES( p_input, p_es );
+                        }
                         break;
                 }
             }
@@ -362,6 +388,7 @@ static void PSInit( input_thread_t * p_input )
  *****************************************************************************/
 static void PSEnd( input_thread_t * p_input )
 {
+    vlc_mutex_destroy( &((packet_cache_t *)p_input->p_plugin_data)->lock );
     free( p_input->p_plugin_data );
 }
 
@@ -432,7 +459,7 @@ static int PSRead( input_thread_t * p_input,
                 /* It is common for MPEG-1 streams to pad with zeros
                  * (although it is forbidden by the recommendation), so
                  * don't bother everybody in this case. */
-                intf_WarnMsg( 1, "Garbage at input (%.8x)", i_startcode );
+                intf_WarnMsg( 3, "Garbage at input (%.8x)", i_startcode );
             }
 
             while( (i_startcode & 0xFFFFFF00) != 0x100L )
@@ -481,8 +508,8 @@ static int PSRead( input_thread_t * p_input,
         }
 
         /* Fetch a packet of the appropriate size. */
-        if( (p_data = NewPacket( p_input->p_method_data, i_packet_size + 6 )) 
-            == NULL )
+        p_data = NewPacket( p_input->p_method_data, i_packet_size + 6 );
+        if( p_data == NULL )
         {
             intf_ErrMsg( "Out of memory" );
             return( -1 );
@@ -571,13 +598,17 @@ static struct data_packet_s * NewPacket( void * p_packet_cache,
         return NULL;
     }
 
+    vlc_mutex_lock( &p_cache->lock );
+
     /* Checks whether the data cache is empty */
     if( p_cache->data.l_index == 0 )
     {
         /* Allocates a new packet */
-        if ( (p_data = malloc( sizeof(data_packet_t) )) == NULL )
+        p_data = malloc( sizeof(data_packet_t) );
+        if( p_data == NULL )
         {
             intf_ErrMsg( "Out of memory" );
+            vlc_mutex_unlock( &p_cache->lock );
             return NULL;
         }
 #ifdef TRACE_INPUT
@@ -591,6 +622,7 @@ static struct data_packet_s * NewPacket( void * p_packet_cache,
             == NULL )
         {
             intf_ErrMsg( "NULL packet in the data cache" );
+            vlc_mutex_unlock( &p_cache->lock );
             return NULL;
         }
     }
@@ -600,13 +632,15 @@ static struct data_packet_s * NewPacket( void * p_packet_cache,
         /* Small buffer */  
    
         /* Checks whether the buffer cache is empty */
-        if( p_cache->small.l_index == 0 )
+        if( p_cache->smallbuffer.l_index == 0 )
         {
-    	    /* Allocates a new packet */
-            if ( (p_data->p_buffer = malloc( l_size )) == NULL )
+            /* Allocates a new packet */
+            p_data->p_buffer = malloc( l_size );
+            if( p_data->p_buffer == NULL )
             {
                 intf_DbgMsg( "Out of memory" );
                 free( p_data );
+                vlc_mutex_unlock( &p_cache->lock );
                 return NULL;
             }
 #ifdef TRACE_INPUT
@@ -617,24 +651,25 @@ static struct data_packet_s * NewPacket( void * p_packet_cache,
         else
         {
             /* Takes the packet out from the cache */
-            l_index = -- p_cache->small.l_index;    
-            if( (p_data->p_buffer = p_cache->small.p_stack[l_index].p_data)
+            l_index = -- p_cache->smallbuffer.l_index;    
+            if( (p_data->p_buffer = p_cache->smallbuffer.p_stack[l_index].p_data)
                 == NULL )
             {
                 intf_ErrMsg( "NULL packet in the small buffer cache" );
                 free( p_data );
+                vlc_mutex_unlock( &p_cache->lock );
                 return NULL;
             }
-	        /* Reallocates the packet if it is too small or too large */
-            if( p_cache->small.p_stack[l_index].l_size < l_size ||
-                p_cache->small.p_stack[l_index].l_size > 2*l_size )
+            /* Reallocates the packet if it is too small or too large */
+            if( p_cache->smallbuffer.p_stack[l_index].l_size < l_size ||
+                p_cache->smallbuffer.p_stack[l_index].l_size > 2*l_size )
             {
                 p_data->p_buffer = realloc( p_data->p_buffer, l_size );
                 p_data->l_size = l_size;
             }
             else
             {
-                p_data->l_size = p_cache->small.p_stack[l_index].l_size;
+                p_data->l_size = p_cache->smallbuffer.p_stack[l_index].l_size;
             }
         }
     }
@@ -643,13 +678,15 @@ static struct data_packet_s * NewPacket( void * p_packet_cache,
         /* Large buffer */  
    
         /* Checks whether the buffer cache is empty */
-        if( p_cache->large.l_index == 0 )
+        if( p_cache->largebuffer.l_index == 0 )
         {
-    	    /* Allocates a new packet */
-            if ( (p_data->p_buffer = malloc( l_size )) == NULL )
+            /* Allocates a new packet */
+            p_data->p_buffer = malloc( l_size );
+            if ( p_data->p_buffer == NULL )
             {
                 intf_ErrMsg( "Out of memory" );
                 free( p_data );
+                vlc_mutex_unlock( &p_cache->lock );
                 return NULL;
             }
 #ifdef TRACE_INPUT
@@ -660,28 +697,31 @@ static struct data_packet_s * NewPacket( void * p_packet_cache,
         else
         {
             /* Takes the packet out from the cache */
-            l_index = -- p_cache->large.l_index;    
-            if( (p_data->p_buffer = p_cache->large.p_stack[l_index].p_data)
-                == NULL )
+            l_index = -- p_cache->largebuffer.l_index;    
+            p_data->p_buffer = p_cache->largebuffer.p_stack[l_index].p_data;
+            if( p_data->p_buffer == NULL )
             {
                 intf_ErrMsg( "NULL packet in the small buffer cache" );
                 free( p_data );
+                vlc_mutex_unlock( &p_cache->lock );
                 return NULL;
             }
-	        /* Reallocates the packet if it is too small or too large */
-            if( p_cache->large.p_stack[l_index].l_size < l_size ||
-                p_cache->large.p_stack[l_index].l_size > 2*l_size )
+            /* Reallocates the packet if it is too small or too large */
+            if( p_cache->largebuffer.p_stack[l_index].l_size < l_size ||
+                p_cache->largebuffer.p_stack[l_index].l_size > 2*l_size )
             {
                 p_data->p_buffer = realloc( p_data->p_buffer, l_size );
                 p_data->l_size = l_size;
             }
             else
             {
-                p_data->l_size = p_cache->large.p_stack[l_index].l_size;
+                p_data->l_size = p_cache->largebuffer.p_stack[l_index].l_size;
             }
-	    }
+        }
     }
 
+    vlc_mutex_unlock( &p_cache->lock );
+
     /* Initialize data */
     p_data->p_next = NULL;
     p_data->b_discard_payload = 0;
@@ -689,7 +729,7 @@ static struct data_packet_s * NewPacket( void * p_packet_cache,
     p_data->p_payload_end = p_data->p_buffer + l_size;
 
     return( p_data );
-	
+
 }
 
 
@@ -711,13 +751,17 @@ static pes_packet_t * NewPES( void * p_packet_cache )
     }
 #endif
 
+    vlc_mutex_lock( &p_cache->lock );	
+
     /* Checks whether the PES cache is empty */
     if( p_cache->pes.l_index == 0 )
     {
         /* Allocates a new packet */
-        if ( (p_pes = malloc( sizeof(pes_packet_t) )) == NULL )
+        p_pes = malloc( sizeof(pes_packet_t) );
+        if( p_pes == NULL )
         {
             intf_DbgMsg( "Out of memory" );
+            vlc_mutex_unlock( &p_cache->lock );	
             return NULL;
         }
 #ifdef TRACE_INPUT
@@ -727,14 +771,17 @@ static pes_packet_t * NewPES( void * p_packet_cache )
     else
     {
         /* Takes the packet out from the cache */
-        if( (p_pes = p_cache->pes.p_stack[ -- p_cache->pes.l_index ]) 
-            == NULL )
+        p_pes = p_cache->pes.p_stack[ -- p_cache->pes.l_index ];
+        if( p_pes == NULL )
         {
             intf_ErrMsg( "NULL packet in the data cache" );
+            vlc_mutex_unlock( &p_cache->lock );
             return NULL;
         }
     }
-	
+
+    vlc_mutex_unlock( &p_cache->lock );
+
     p_pes->b_data_alignment = p_pes->b_discontinuity =
         p_pes->i_pts = p_pes->i_dts = 0;
     p_pes->i_pes_size = 0;
@@ -762,7 +809,9 @@ static void DeletePacket( void * p_packet_cache,
     }
 #endif
 
-	ASSERT( p_data );
+    ASSERT( p_data );
+
+    vlc_mutex_lock( &p_cache->lock );
 
     /* Checks whether the data cache is full */
     if ( p_cache->data.l_index < DATA_CACHE_SIZE )
@@ -773,12 +822,12 @@ static void DeletePacket( void * p_packet_cache,
         if ( p_data->l_size < MAX_SMALL_SIZE )
         {
             /* Checks whether the small buffer cache is full */
-            if ( p_cache->small.l_index < SMALL_CACHE_SIZE )
+            if ( p_cache->smallbuffer.l_index < SMALL_CACHE_SIZE )
             {
-                p_cache->small.p_stack[ p_cache->small.l_index ].l_size = 
-                    p_data->l_size;
-                p_cache->small.p_stack[ p_cache->small.l_index ++ ].p_data = 
-                    p_data->p_buffer;
+                p_cache->smallbuffer.p_stack[
+                    p_cache->smallbuffer.l_index ].l_size = p_data->l_size;
+                p_cache->smallbuffer.p_stack[
+                    p_cache->smallbuffer.l_index++ ].p_data = p_data->p_buffer;
             }
             else
             {
@@ -789,15 +838,15 @@ static void DeletePacket( void * p_packet_cache,
 #endif
             }
         }
-    	else
+        else
         {
             /* Checks whether the large buffer cache is full */
-            if ( p_cache->large.l_index < LARGE_CACHE_SIZE )
+            if ( p_cache->largebuffer.l_index < LARGE_CACHE_SIZE )
             {
-                p_cache->large.p_stack[ p_cache->large.l_index ].l_size = 
-                    p_data->l_size;
-                p_cache->large.p_stack[ p_cache->large.l_index ++ ].p_data = 
-                    p_data->p_buffer;
+                p_cache->largebuffer.p_stack[
+                    p_cache->largebuffer.l_index ].l_size = p_data->l_size;
+                p_cache->largebuffer.p_stack[
+                    p_cache->largebuffer.l_index++ ].p_data = p_data->p_buffer;
             }
             else
             {
@@ -813,12 +862,13 @@ static void DeletePacket( void * p_packet_cache,
     {
         /* Cache full: the packet must be freed */
         free( p_data->p_buffer );
-    	free( p_data );
+        free( p_data );
 #ifdef TRACE_INPUT
         intf_DbgMsg( "PS input: data packet freed" );
 #endif
     }
 
+    vlc_mutex_unlock( &p_cache->lock );
 }
 
 /*****************************************************************************
@@ -851,10 +901,12 @@ static void DeletePES( void * p_packet_cache, pes_packet_t * p_pes )
         p_data = p_next;
     }
 
+    vlc_mutex_lock( &p_cache->lock );
+
     /* Checks whether the PES cache is full */
     if ( p_cache->pes.l_index < PES_CACHE_SIZE )
     {
-    	/* Cache not full: store the packet in it */
+        /* Cache not full: store the packet in it */
         p_cache->pes.p_stack[ p_cache->pes.l_index ++ ] = p_pes;
     }
     else
@@ -865,5 +917,7 @@ static void DeletePES( void * p_packet_cache, pes_packet_t * p_pes )
         intf_DbgMsg( "PS input: PES packet freed" );
 #endif
     }
+
+    vlc_mutex_unlock( &p_cache->lock );
 }
 
