@@ -3,7 +3,7 @@
  * Provides functions to perform the YUV conversion.
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: video_yuvmmx.c,v 1.8 2001/04/15 04:19:58 sam Exp $
+ * $Id: video_yuvmmx.c,v 1.10 2001/06/03 12:47:21 sam Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -42,8 +42,6 @@
 #include "mtime.h"
 #include "tests.h"
 
-#include "modules.h"
-
 #include "video.h"
 #include "video_output.h"
 
@@ -51,6 +49,9 @@
 
 #include "intf_msg.h"
 
+#include "modules.h"
+#include "modules_export.h"
+
 static int     yuv_Probe      ( probedata_t *p_data );
 static int     yuv_Init       ( vout_thread_t *p_vout );
 static int     yuv_Reset      ( vout_thread_t *p_vout );
@@ -79,21 +80,17 @@ void _M( yuv_getfunctions )( function_list_t * p_function_list )
 static int yuv_Probe( probedata_t *p_data )
 {
     /* Test for MMX support in the CPU */
-    if( TestCPU( CPU_CAPABILITY_MMX ) )
-    {
-        if( TestMethod( YUV_METHOD_VAR, "yuvmmx" ) )
-        {
-            return( 999 );
-        }
-        else
-        {
-            return( 100 );
-        }
-    }
-    else
+    if( !TestCPU( CPU_CAPABILITY_MMX ) )
     {
         return( 0 );
     }
+
+    if( TestMethod( YUV_METHOD_VAR, "yuvmmx" ) )
+    {
+        return( 999 );
+    }
+
+    return( 100 );
 }
 
 /*****************************************************************************
