@@ -4,7 +4,7 @@
  * decoders.
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: input.c,v 1.98 2001/04/15 04:19:58 sam Exp $
+ * $Id: input.c,v 1.118 2001/05/31 12:45:39 sam Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -32,19 +32,29 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <unistd.h>
+
+#ifdef HAVE_UNISTD_H
+#   include <unistd.h>
+#elif defined( _MSC_VER ) && defined( _WIN32 )
+#   include <io.h>
+#endif
+
 #include <string.h>
 #include <errno.h>
 
-/* Network functions */
+#ifdef STRNCASECMP_IN_STRINGS_H
+#   include <strings.h>
+#endif
 
-#if !defined( SYS_BEOS ) && !defined( SYS_NTO )
-#include <netdb.h>                                             /* hostent ... */
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <sys/types.h>
-#include <sys/socket.h>
+#ifdef WIN32
+#   include <winsock2.h>
+#elif !defined( SYS_BEOS ) && !defined( SYS_NTO )
+#   include <netdb.h>                                         /* hostent ... */
+#   include <sys/socket.h>
+#   include <netinet/in.h>
+#   include <arpa/inet.h>
+#   include <sys/types.h>
+#   include <sys/socket.h>
 #endif
 
 #ifdef STATS
@@ -70,9 +80,6 @@
 
 #include "main.h"
 
- /* #include <netutils.h> */
-
-
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
@@ -82,6 +89,13 @@ static void ErrorThread     ( input_thread_t *p_input );
 static void DestroyThread   ( input_thread_t *p_input );
 static void EndThread       ( input_thread_t *p_input );
 
+static void FileOpen        ( input_thread_t *p_input );
+static void FileClose       ( input_thread_t *p_input );
+#if !defined( SYS_BEOS ) && !defined( SYS_NTO )
+static void NetworkOpen     ( input_thread_t *p_input );
+static void NetworkClose    ( input_thread_t *p_input );
+#endif
+
 /*****************************************************************************
  * input_CreateThread: creates a new input thread
  *****************************************************************************
@@ -130,6 +144,7 @@ input_thread_t *input_CreateThread ( playlist_item_t *p_item, int *pi_status )
     p_input->stream.i_area_nb = 0;
     p_input->stream.pp_areas = NULL;
     p_input->stream.p_selected_area = NULL;
+    p_input->stream.p_new_area = NULL;
     /* By default there is one areas in a stream */
     input_AddArea( p_input );
     p_input->stream.p_selected_area = p_input->stream.pp_areas[0];
@@ -140,9 +155,13 @@ input_thread_t *input_CreateThread ( playlist_item_t *p_item, int *pi_status )
     p_input->stream.control.b_mute = 0;
     p_input->stream.control.b_bw = 0;
 
-    /* Initialize default settings for spawned decoders */
-    p_input->p_default_aout = p_main->p_aout;
-    p_input->p_default_vout = p_main->p_vout;
+    /* Setup callbacks */
+    p_input->pf_file_open     = FileOpen;
+    p_input->pf_file_close    = FileClose;
+#if !defined( SYS_BEOS ) && !defined( SYS_NTO )
+    p_input->pf_network_open  = NetworkOpen;
+    p_input->pf_network_close = NetworkClose;
+#endif
 
     /* Create thread and set locks. */
     vlc_mutex_init( &p_input->stream.stream_lock );
@@ -213,6 +232,7 @@ void input_DestroyThread( input_thread_t *p_input, int *pi_status )
 static void RunThread( input_thread_t *p_input )
 {
     int                     i_error, i;
+    data_packet_t **        pp_packets;
 
     if( InitThread( p_input ) )
     {
@@ -225,16 +245,34 @@ static void RunThread( input_thread_t *p_input )
         return;
     }
 
+    /* initialization is completed */
+    vlc_mutex_lock( &p_input->stream.stream_lock );
+    p_input->stream.b_changed = 1;
+    vlc_mutex_unlock( &p_input->stream.stream_lock );
+
+    pp_packets = (data_packet_t **) malloc( p_input->i_read_once *
+                                        sizeof( data_packet_t * ) );
+    if( pp_packets == NULL )
+    {
+        intf_ErrMsg( "input error: out of memory" );
+        free( pp_packets );
+        p_input->b_error = 1;
+    }
+
     while( !p_input->b_die && !p_input->b_error && !p_input->b_eof )
     {
-        data_packet_t *         pp_packets[p_input->i_read_once];
-
 #ifdef STATS
         p_input->c_loops++;
 #endif
 
         vlc_mutex_lock( &p_input->stream.stream_lock );
 
+        if( p_input->stream.p_new_area )
+        {
+            p_input->pf_set_area( p_input, p_input->stream.p_new_area );
+            p_input->stream.p_new_area = NULL;
+        }
+
         if( p_input->stream.p_selected_area->i_seek != NO_SEEK )
         {
             if( p_input->stream.b_seekable && p_input->pf_seek != NULL )
@@ -257,6 +295,18 @@ static void RunThread( input_thread_t *p_input )
             p_input->stream.p_selected_area->i_seek = NO_SEEK;
         }
 
+        if( p_input->stream.p_removed_es )
+        {
+            input_UnselectES( p_input, p_input->stream.p_removed_es );
+            p_input->stream.p_removed_es = NULL;
+        }
+
+        if( p_input->stream.p_newly_selected_es )
+        {
+            input_SelectES( p_input, p_input->stream.p_newly_selected_es );
+            p_input->stream.p_newly_selected_es = NULL;
+        }
+
         vlc_mutex_unlock( &p_input->stream.stream_lock );
 
         i_error = p_input->pf_read( p_input, pp_packets );
@@ -273,7 +323,7 @@ static void RunThread( input_thread_t *p_input )
             {
                 /* End of file - we do not set b_die because only the
                  * interface is allowed to do so. */
-                intf_WarnMsg( 1, "input: EOF reached" );
+                intf_WarnMsg( 3, "input: EOF reached" );
                 p_input->b_eof = 1;
             }
             else
@@ -283,6 +333,8 @@ static void RunThread( input_thread_t *p_input )
         }
     }
 
+    free( pp_packets );
+
     if( p_input->b_error || p_input->b_eof )
     {
         ErrorThread( p_input );
@@ -310,8 +362,11 @@ static int InitThread( input_thread_t * p_input )
     p_input->c_packets_trashed          = 0;
 #endif
 
-    p_input->p_input_module = module_Need( p_main->p_bank,
-                                           MODULE_CAPABILITY_INPUT,
+    /* Default, might get overwritten */
+    p_input->pf_open = p_input->pf_file_open;
+    p_input->pf_close = p_input->pf_file_close;
+
+    p_input->p_input_module = module_Need( MODULE_CAPABILITY_INPUT,
                                            (probedata_t *)p_input );
 
     if( p_input->p_input_module == NULL )
@@ -323,8 +378,14 @@ static int InitThread( input_thread_t * p_input )
 
 #define f p_input->p_input_module->p_functions->input.functions.input
     p_input->pf_init          = f.pf_init;
-    p_input->pf_open          = f.pf_open;
-    p_input->pf_close         = f.pf_close;
+    if( f.pf_open != NULL )
+    {
+        p_input->pf_open          = f.pf_open;
+    }
+    if( f.pf_close != NULL )
+    {
+        p_input->pf_close         = f.pf_close;
+    }
     p_input->pf_end           = f.pf_end;
     p_input->pf_read          = f.pf_read;
     p_input->pf_set_area      = f.pf_set_area;
@@ -342,7 +403,7 @@ static int InitThread( input_thread_t * p_input )
     {
         /* We barfed -- exit nicely */
         p_input->pf_close( p_input );
-        module_Unneed( p_main->p_bank, p_input->p_input_module );
+        module_Unneed( p_input->p_input_module );
         return( -1 );
     }
 
@@ -352,7 +413,7 @@ static int InitThread( input_thread_t * p_input )
     {
         /* We barfed -- exit nicely */
         p_input->pf_close( p_input );
-        module_Unneed( p_main->p_bank, p_input->p_input_module );
+        module_Unneed( p_input->p_input_module );
         return( -1 );
     }
 
@@ -406,7 +467,7 @@ static void EndThread( input_thread_t * p_input )
     p_input->pf_close( p_input );
 
     /* Release modules */
-    module_Unneed( p_main->p_bank, p_input->p_input_module );
+    module_Unneed( p_input->p_input_module );
 
 }
 
@@ -432,13 +493,17 @@ static void DestroyThread( input_thread_t * p_input )
 }
 
 /*****************************************************************************
- * input_FileOpen : open a file descriptor
+ * FileOpen : open a file descriptor
  *****************************************************************************/
-void input_FileOpen( input_thread_t * p_input )
+static void FileOpen( input_thread_t * p_input )
 {
     struct stat         stat_info;
     int                 i_stat;
 
+#if defined( WIN32 )
+    char buf[7] = { 0 };
+#endif
+
     char *psz_name = p_input->p_source;
 
     /* FIXME: this code ought to be in the plugin so that code can
@@ -453,6 +518,9 @@ void input_FileOpen( input_thread_t * p_input )
             /* get rid of the 'dvd:' stuff and try again */
             psz_name += 4;
             i_stat = stat( psz_name, &stat_info );
+#if defined( WIN32 )
+            _snprintf( buf, 7, "\\\\.\\%c:", psz_name[0] );
+#endif
         }
         else if( ( i_size > 5 )
                  && !strncasecmp( psz_name, "file:", 5 ) )
@@ -462,7 +530,11 @@ void input_FileOpen( input_thread_t * p_input )
             i_stat = stat( psz_name, &stat_info );
         }
 
-        if( i_stat == (-1) )
+        if( i_stat == (-1) 
+#if defined( WIN32 )
+        && !buf[0]      
+#endif
+            )
         {
             intf_ErrMsg( "input error: cannot stat() file `%s' (%s)",
                          psz_name, strerror(errno));
@@ -477,13 +549,17 @@ void input_FileOpen( input_thread_t * p_input )
     p_input->stream.b_pace_control = 1;
 
     if( S_ISREG(stat_info.st_mode) || S_ISCHR(stat_info.st_mode)
-         || S_ISBLK(stat_info.st_mode) )
+         || S_ISBLK(stat_info.st_mode)
+#if defined( WIN32 )
+         || ( buf[0] && ( ( stat_info.st_size = 0 ) == 0 ) )
+#endif
+         )
     {
         p_input->stream.b_seekable = 1;
         p_input->stream.p_selected_area->i_size = stat_info.st_size;
     }
     else if( S_ISFIFO(stat_info.st_mode)
-#ifndef SYS_BEOS
+#if !defined( SYS_BEOS ) && !defined( WIN32 )
              || S_ISSOCK(stat_info.st_mode)
 #endif
              )
@@ -503,9 +579,16 @@ void input_FileOpen( input_thread_t * p_input )
     p_input->stream.p_selected_area->i_tell = 0;
     vlc_mutex_unlock( &p_input->stream.stream_lock );
 
-    intf_Msg( "input: opening %s", p_input->p_source );
+    intf_WarnMsg( 1, "input: opening file `%s'", p_input->p_source );
+#ifndef WIN32
     if( (p_input->i_handle = open( psz_name,
                                    /*O_NONBLOCK | O_LARGEFILE*/0 )) == (-1) )
+#else
+    if( ( buf[0] && ( (HANDLE) p_input->i_handle = CreateFile( buf, 
+        GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, 
+        NULL, OPEN_EXISTING, 0, NULL ) ) == INVALID_HANDLE_VALUE ) ||
+        ( !buf[0] && (p_input->i_handle = open( psz_name, O_BINARY ) ) == (-1) ) )
+#endif
     {
         intf_ErrMsg( "input error: cannot open file (%s)", strerror(errno) );
         p_input->b_error = 1;
@@ -515,54 +598,164 @@ void input_FileOpen( input_thread_t * p_input )
 }
 
 /*****************************************************************************
- * input_FileClose : close a file descriptor
+ * FileClose : close a file descriptor
  *****************************************************************************/
-void input_FileClose( input_thread_t * p_input )
+static void FileClose( input_thread_t * p_input )
 {
-    intf_Msg( "input: closing %s", p_input->p_source );
+    intf_WarnMsg( 1, "input: closing file `%s'", p_input->p_source );
+#if defined( WIN32 )
+    if( ( strlen( p_input->p_source ) > 4 ) &&
+        !strncasecmp( p_input->p_source, "dvd:", 4 ) )
+    {
+        CloseHandle( (HANDLE) p_input->i_handle );
+    }
+    else
+#endif
     close( p_input->i_handle );
 
     return;
 }
 
-
 #if !defined( SYS_BEOS ) && !defined( SYS_NTO )
 /*****************************************************************************
- * input_NetworkOpen : open a network socket 
+ * NetworkOpen : open a network socket 
  *****************************************************************************/
-void input_NetworkOpen( input_thread_t * p_input )
+static void NetworkOpen( input_thread_t * p_input )
 {
- 
-    int                 i_option_value, i_port;
-    struct sockaddr_in  s_socket;
-    boolean_t           b_broadcast;
-    
-    /* FIXME : we don't handle channels for the moment */
+    char                *psz_server = NULL;
+    char                *psz_broadcast = NULL;
+    int                 i_port = 0;
+    int                 i_opt;
+    struct sockaddr_in  sock;
+
+#ifdef WIN32
+    /* WinSock Library Init. */
+    WSADATA Data;
+    int i_err = WSAStartup( MAKEWORD( 1, 1 ), &Data );
+
+    if( i_err )
+    {
+        intf_ErrMsg( "input: can't initiate WinSocks, error %i", i_err );
+        return ;
+    }
+#endif
     
     /* Get the remote server */
-    if( p_input->p_source == NULL )
+    if( p_input->p_source != NULL )
     {
-        p_input->p_source = main_GetPszVariable( INPUT_SERVER_VAR, 
-                                                 INPUT_SERVER_DEFAULT );
+        psz_server = p_input->p_source;
+
+        /* Skip the protocol name */
+        while( *psz_server && *psz_server != ':' )
+        {
+            psz_server++;
+        }
+
+        /* Skip the "://" part */
+        while( *psz_server && (*psz_server == ':' || *psz_server == '/') )
+        {
+            psz_server++;
+        }
+
+        /* Found a server name */
+        if( *psz_server )
+        {
+            char *psz_port = psz_server;
+
+            /* Skip the hostname part */
+            while( *psz_port && *psz_port != ':' )
+            {
+                psz_port++;
+            }
+
+            /* Found a port name */
+            if( *psz_port )
+            {
+                /* Replace ':' with '\0' */
+                *psz_port = '\0';
+                psz_port++;
+
+                psz_broadcast = psz_port;
+                while( *psz_broadcast && *psz_broadcast != ':' )
+                {
+                    psz_broadcast++;
+                }
+
+                if( *psz_broadcast )
+                {
+                    *psz_broadcast = '\0';
+                    psz_broadcast++;
+                    while( *psz_broadcast && *psz_broadcast == ':' )
+                    {
+                        psz_broadcast++;
+                    }
+                }
+                else
+                {
+                    psz_broadcast = NULL;
+                }
+
+                /* port before broadcast address */
+                if( *psz_port != ':' )
+                {
+                    i_port = atoi( psz_port );
+                }
+            }
+        }
+        else
+        {
+            psz_server = NULL;
+        }
     }
-    
+
+    /* Check that we got a valid server */
+    if( psz_server == NULL )
+    {
+        psz_server = main_GetPszVariable( INPUT_SERVER_VAR, 
+                                          INPUT_SERVER_DEFAULT );
+    }
+
+    /* Check that we got a valid port */
+    if( i_port == 0 )
+    {
+        i_port = main_GetIntVariable( INPUT_PORT_VAR, INPUT_PORT_DEFAULT );
+    }
+
+    if( psz_broadcast == NULL )
+    {
+        /* Are we broadcasting ? */
+        if( main_GetIntVariable( INPUT_BROADCAST_VAR,
+                                 INPUT_BROADCAST_DEFAULT ) )
+        {
+            psz_broadcast = main_GetPszVariable( INPUT_BCAST_ADDR_VAR,
+                                                 INPUT_BCAST_ADDR_DEFAULT );
+        }
+        else
+        {
+           psz_broadcast = NULL; 
+        }
+    }
+
+    intf_WarnMsg( 2, "input: server: %s port: %d broadcast: %s",
+                     psz_server, i_port, psz_broadcast );
+
     /* Open a SOCK_DGRAM (UDP) socket, in the AF_INET domain, automatic (0)
      * protocol */
     p_input->i_handle = socket( AF_INET, SOCK_DGRAM, 0 );
     if( p_input->i_handle == -1 )
     {
-        intf_ErrMsg("NetworkOpen : can't create socket : %s", strerror(errno));
+        intf_ErrMsg("input error: can't create socket : %s", strerror(errno));
         p_input->b_error = 1;
         return;
     }
 
     /* We may want to reuse an already used socket */
-    i_option_value = 1;
+    i_opt = 1;
     if( setsockopt( p_input->i_handle, SOL_SOCKET, SO_REUSEADDR,
-                    &i_option_value,sizeof( i_option_value ) ) == -1 )
+                    (void*) &i_opt, sizeof( i_opt ) ) == -1 )
     {
-        intf_ErrMsg("NetworkOpen : can't configure socket (SO_REUSEADDR: %s)",
-                    strerror(errno));
+        intf_ErrMsg( "input error: can't configure socket (SO_REUSEADDR: %s)",
+                     strerror(errno));
         close( p_input->i_handle );
         p_input->b_error = 1;
         return;
@@ -570,56 +763,51 @@ void input_NetworkOpen( input_thread_t * p_input )
 
     /* Increase the receive buffer size to 1/2MB (8Mb/s during 1/2s) to avoid
      * packet loss caused by scheduling problems */
-    i_option_value = 524288;
-    if( setsockopt( p_input->i_handle, SOL_SOCKET, SO_RCVBUF, &i_option_value,
-                    sizeof( i_option_value ) ) == -1 )
+    i_opt = 0x80000;
+    if( setsockopt( p_input->i_handle, SOL_SOCKET, SO_RCVBUF,
+                    (void*) &i_opt, sizeof( i_opt ) ) == -1 )
     {
-        intf_ErrMsg("NetworkOpen : can't configure socket (SO_RCVBUF: %s)", 
-                    strerror(errno));
+        intf_ErrMsg( "input error: can't configure socket (SO_RCVBUF: %s)", 
+                     strerror(errno));
         close( p_input->i_handle );
         p_input->b_error = 1;
         return;
     }
 
-    /* Get details about what we are supposed to do */
-    b_broadcast = (boolean_t)main_GetIntVariable( INPUT_BROADCAST_VAR, 0 );
-    i_port = main_GetIntVariable( INPUT_PORT_VAR, INPUT_PORT_DEFAULT );
-
-    /* TODO : here deal with channel stufs */
-    
     /* Build the local socket */
-    if ( network_BuildLocalAddr( &s_socket, i_port, b_broadcast ) 
-         == -1 )
+    if ( network_BuildLocalAddr( &sock, i_port, psz_broadcast ) == -1 )
     {
+        intf_ErrMsg( "input error: can't build local address" );
         close( p_input->i_handle );
         p_input->b_error = 1;
         return;
     }
     
     /* Bind it */
-    if( bind( p_input->i_handle, (struct sockaddr *)&s_socket, 
-              sizeof( s_socket ) ) < 0 )
+    if( bind( p_input->i_handle, (struct sockaddr *)&sock, 
+              sizeof( sock ) ) < 0 )
     {
-        intf_ErrMsg("NetworkOpen: can't bind socket (%s)", strerror(errno));
+        intf_ErrMsg("input error: can't bind socket (%s)", strerror(errno));
         close( p_input->i_handle );
         p_input->b_error = 1;
         return;
     }
 
     /* Build socket for remote connection */
-    if ( network_BuildRemoteAddr( &s_socket, p_input->p_source ) 
-         == -1 )
+    if ( network_BuildRemoteAddr( &sock, psz_server ) == -1 )
     {
+        intf_ErrMsg( "input error: can't build remote address" );
         close( p_input->i_handle );
         p_input->b_error = 1;
         return;
     }
 
     /* And connect it ... should we really connect ? */
-    if( connect( p_input->i_handle, (struct sockaddr *) &s_socket,
-                 sizeof( s_socket ) ) == (-1) )
+    if( connect( p_input->i_handle, (struct sockaddr *) &sock,
+                 sizeof( sock ) ) == (-1) )
     {
-        intf_ErrMsg("NetworkOpen: can't connect socket" );
+        intf_ErrMsg( "input error: can't connect socket, %s", 
+                     strerror(errno) );
         close( p_input->i_handle );
         p_input->b_error = 1;
         return;
@@ -627,17 +815,25 @@ void input_NetworkOpen( input_thread_t * p_input )
 
     /* We can't pace control, but FIXME : bug in meuuh's code to sync PCR
      * with the server. */
-    p_input->stream.b_pace_control = 1;
+    p_input->stream.b_pace_control = 0;
+    p_input->stream.b_seekable = 0;
+
+    intf_WarnMsg( 3, "input: successfully opened network mode" );
     
     return;
 }
 
 /*****************************************************************************
- * input_NetworkClose : close a network socket
+ * NetworkClose : close a network socket
  *****************************************************************************/
-void input_NetworkClose( input_thread_t * p_input )
+static void NetworkClose( input_thread_t * p_input )
 {
     close( p_input->i_handle );
-    /* FIXME: deal with channels */
+
+#ifdef WIN32 
+    WSACleanup();
+#endif
+
 }
 #endif
+
