@@ -4,7 +4,7 @@
  * interface, such as command line.
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: interface.c,v 1.73 2001/04/28 03:36:25 sam Exp $
+ * $Id: interface.c,v 1.79 2001/05/15 14:49:48 stef Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -47,7 +47,6 @@
 
 #include "intf_msg.h"
 #include "interface.h"
-#include "intf_cmd.h"
 #include "intf_playlist.h"
 #include "intf_channels.h"
 #include "keystrokes.h"
@@ -82,8 +81,7 @@ intf_thread_t* intf_Create( void )
     }
 
     /* Choose the best module */
-    p_intf->p_module = module_Need( p_main->p_bank,
-                                    MODULE_CAPABILITY_INTF, NULL );
+    p_intf->p_module = module_Need( MODULE_CAPABILITY_INTF, NULL );
 
     if( p_intf->p_module == NULL )
     {
@@ -112,7 +110,7 @@ intf_thread_t* intf_Create( void )
     if( p_intf->pf_open( p_intf ) )
     {
         intf_ErrMsg("intf error: cannot create interface");
-        module_Unneed( p_main->p_bank, p_intf->p_module );
+        module_Unneed( p_intf->p_module );
         free( p_intf );
         return( NULL );
     }
@@ -126,7 +124,7 @@ intf_thread_t* intf_Create( void )
     intf_LoadChannels( p_intf, main_GetPszVariable( INTF_CHANNELS_VAR,
                                                     INTF_CHANNELS_DEFAULT ));
 
-    intf_Msg("intf: interface initialized");
+    intf_WarnMsg( 1, "intf: interface initialized");
     return( p_intf );
 }
 
@@ -143,7 +141,7 @@ static void intf_Manage( intf_thread_t *p_intf )
     intf_FlushMsg();
 
     /* Manage module bank */
-    module_ManageBank( p_main->p_bank );
+    module_ManageBank( );
 
     if( ( p_intf->p_input != NULL ) &&
             ( p_intf->p_input->b_error || p_intf->p_input->b_eof ) )
@@ -156,21 +154,45 @@ static void intf_Manage( intf_thread_t *p_intf )
     /* If no stream is being played, try to find one */
     if( p_intf->p_input == NULL && !p_intf->b_die )
     {
-        /* Select the next playlist item */
-        intf_PlaylistNext( p_main->p_playlist );
+//        vlc_mutex_lock( &p_main->p_playlist->change_lock );
 
-        if( p_main->p_playlist->i_index == -1 )
+        if( !p_main->p_playlist->b_stopped )
         {
-            /*    FIXME: wait for user to add stuff to playlist ? */
-#if 0
-            p_intf->b_die = 1;
-#endif
+            /* Select the next playlist item */
+            intf_PlaylistNext( p_main->p_playlist );
+
+            /* don't loop by default: stop at playlist end */
+            if( p_main->p_playlist->i_index == -1 )
+            {
+                p_main->p_playlist->b_stopped = 1;
+            }
+            else
+            {
+                p_main->p_playlist->b_stopped = 0;
+                p_main->p_playlist->i_mode = PLAYLIST_FORWARD + 
+                    main_GetIntVariable( PLAYLIST_LOOP_VAR,
+                                         PLAYLIST_LOOP_DEFAULT );
+                p_intf->p_input =
+                    input_CreateThread( &p_main->p_playlist->current, NULL );
+            }
         }
         else
         {
-            p_intf->p_input =
-                input_CreateThread( &p_main->p_playlist->current, NULL );
+            /* playing has been stopped: we no longer need outputs */
+            if( p_aout_bank->i_count )
+            {
+                /* FIXME kludge that does not work with several outputs */
+                aout_DestroyThread( p_aout_bank->pp_aout[0], NULL );
+                p_aout_bank->i_count--;
+            }
+            if( p_vout_bank->i_count )
+            {
+                vout_DestroyThread( p_vout_bank->pp_vout[0], NULL );
+                p_vout_bank->i_count--;
+            }
         }
+
+//        vlc_mutex_unlock( &p_main->p_playlist->change_lock );
     }
 }
 
@@ -206,7 +228,7 @@ void intf_Destroy( intf_thread_t *p_intf )
     }
          
     /* Unlock module */
-    module_Unneed( p_main->p_bank, p_intf->p_module );
+    module_Unneed( p_intf->p_module );
 
     vlc_mutex_destroy( &p_intf->change_lock );
 
@@ -319,7 +341,7 @@ void intf_AssignNormalKeys( intf_thread_t *p_intf)
  *****************************************************************************/
 int intf_ProcessKey( intf_thread_t *p_intf, int g_key )
 {
-    static int i_volbackup;
+    int i_index;
     keyparm k_reply;
     
     k_reply = intf_GetKey( p_intf, g_key); 
@@ -328,6 +350,7 @@ int intf_ProcessKey( intf_thread_t *p_intf, int g_key )
     case INTF_KEY_QUIT:                                        /* quit order */
         p_intf->b_die = 1;
         break;
+
     case INTF_KEY_SET_CHANNEL:
         /* Change channel - return code is ignored since SelectChannel displays
          * its own error messages */
@@ -336,23 +359,62 @@ int intf_ProcessKey( intf_thread_t *p_intf, int g_key )
 /* FIXME : keyboard event is for the time being half handled by the interface
  * half handled directly by the plugins. We should decide what to do. */        
         break;
+
     case INTF_KEY_INC_VOLUME:                                    /* volume + */
-        if( (p_main->p_aout != NULL) && (p_main->p_aout->vol < VOLUME_MAX) )
-            p_main->p_aout->vol += VOLUME_STEP;
-        break;
-    case INTF_KEY_DEC_VOLUME:                                    /* volume - */
-        if( (p_main->p_aout != NULL) && (p_main->p_aout->vol > VOLUME_STEP) )
-            p_main->p_aout->vol -= VOLUME_STEP;
-        break;
-    case INTF_KEY_TOGGLE_VOLUME:                              /* toggle mute */
-        if( (p_main->p_aout != NULL) && (p_main->p_aout->vol))
+        vlc_mutex_lock( &p_aout_bank->lock );
+        for( i_index = 0 ; i_index < p_aout_bank->i_count ; i_index++ )
         {
-            i_volbackup = p_main->p_aout->vol;
-            p_main->p_aout->vol = 0;
+            if( p_aout_bank->pp_aout[i_index]->i_volume
+                                                   < VOLUME_MAX - VOLUME_STEP )
+            {
+                p_aout_bank->pp_aout[i_index]->i_volume += VOLUME_STEP;
+            }
+            else
+            {
+                p_aout_bank->pp_aout[i_index]->i_volume = VOLUME_MAX;
+            }
         }
-        else if( (p_main->p_aout != NULL) && (!p_main->p_aout->vol))
-            p_main->p_aout->vol = i_volbackup;
+        vlc_mutex_unlock( &p_aout_bank->lock );
         break;
+
+    case INTF_KEY_DEC_VOLUME:                                    /* volume - */
+        vlc_mutex_lock( &p_aout_bank->lock );
+        for( i_index = 0 ; i_index < p_aout_bank->i_count ; i_index++ )
+        {
+            if( p_aout_bank->pp_aout[i_index]->i_volume > VOLUME_STEP )
+            {
+                p_aout_bank->pp_aout[i_index]->i_volume -= VOLUME_STEP;
+            }
+            else
+            {
+                p_aout_bank->pp_aout[i_index]->i_volume = 0;
+            }
+        }
+        vlc_mutex_unlock( &p_aout_bank->lock );
+        break;
+
+    case INTF_KEY_TOGGLE_VOLUME:                              /* toggle mute */
+        vlc_mutex_lock( &p_aout_bank->lock );
+        for( i_index = 0 ; i_index < p_aout_bank->i_count ; i_index++ )
+        {
+            if( p_aout_bank->pp_aout[i_index]->i_savedvolume )
+            {
+                p_aout_bank->pp_aout[i_index]->i_volume =
+                                p_aout_bank->pp_aout[i_index]->i_savedvolume;
+                p_aout_bank->pp_aout[i_index]->i_savedvolume = 0;
+            }
+            else
+            {
+                p_aout_bank->pp_aout[i_index]->i_savedvolume =
+                                p_aout_bank->pp_aout[i_index]->i_volume;
+                p_aout_bank->pp_aout[i_index]->i_volume = 0;
+            }
+        }
+        vlc_mutex_unlock( &p_aout_bank->lock );
+        break;
+
+/* XXX: fix this later */
+#if 0
     case INTF_KEY_DEC_GAMMA:                                      /* gamma - */
         if( (p_main->p_vout != NULL) && (p_main->p_vout->f_gamma > -INTF_GAMMA_LIMIT) )
         {
@@ -369,6 +431,8 @@ int intf_ProcessKey( intf_thread_t *p_intf, int g_key )
             p_main->p_vout->i_changes |= VOUT_GAMMA_CHANGE;
         }
         break;
+#endif
+
    case INTF_KEY_DUMP_STREAM:
         if( p_intf->p_input != NULL )
         {
@@ -377,9 +441,11 @@ int intf_ProcessKey( intf_thread_t *p_intf, int g_key )
             vlc_mutex_unlock( &p_intf->p_input->stream.stream_lock );
         }
         break;
-   default:                                                   /* unknown key */
+
+    default:                                                  /* unknown key */
         return( 1 );
     }
 
     return( 0 );
 }
+
