@@ -2,7 +2,7 @@
  * vpar_blocks.c : blocks parsing
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: vpar_blocks.c,v 1.76 2001/02/13 13:01:15 massiot Exp $
+ * $Id: vpar_blocks.c,v 1.81 2001/05/06 04:32:03 sam Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Jean-Marc Dressler <polux@via.ecp.fr>
@@ -45,13 +45,12 @@
 
 #include "video_decoder.h"
 #include "vdec_motion.h"
-#include "../video_decoder/vdec_idct.h"
 
 #include "vpar_blocks.h"
-#include "../video_decoder/vpar_headers.h"
-#include "../video_decoder/vpar_synchro.h"
-#include "../video_decoder/video_parser.h"
-#include "../video_decoder/video_fifo.h"
+#include "vpar_headers.h"
+#include "vpar_synchro.h"
+#include "video_parser.h"
+#include "video_fifo.h"
 
 /*
  * Welcome to vpar_blocks.c ! Here's where the heavy processor-critical parsing
@@ -204,7 +203,8 @@ static lookup_t     pl_dct_dc_chrom_init_table_2[32] =
     };
  
 
-    /* Tables for ac DCT coefficients. There are cut in many parts to save space */
+    /* Tables for ac DCT coefficients. There are cut in many parts to
+     * save space */
     /* Table B-14, DCT coefficients table zero,
      * codes 0100 ... 1xxx (used for first (DC) coefficient)
      */
@@ -627,10 +627,6 @@ static __inline__ void DecodeMPEG1NonIntra( vpar_thread_t * p_vpar,
     boolean_t       b_sign;
     dct_lookup_t *  p_tab;
 
-    /* There should be no D picture in non-intra blocks */
-    if( p_vpar->picture.i_coding_type == D_CODING_TYPE )
-        intf_ErrMsg("vpar error : D-picture in non intra block");
-    
     /* Decoding of the AC coefficients */
 
     i_nc = 0;
@@ -710,7 +706,7 @@ static __inline__ void DecodeMPEG1NonIntra( vpar_thread_t * p_vpar,
         p_mb->ppi_blocks[i_b][i_pos] = i_level;
     }
 
-    intf_ErrMsg("vpar error: DCT coeff (non-intra) is out of bounds");
+    intf_WarnMsg( 2, "DCT coeff (non-intra) is out of bounds" );
     p_vpar->picture.b_error = 1;
 }
 
@@ -858,7 +854,7 @@ static __inline__ void DecodeMPEG1Intra( vpar_thread_t * p_vpar,
         p_mb->ppi_blocks[i_b][i_pos] = i_level;
     }
 
-    intf_ErrMsg("vpar error: DCT coeff (intra) is out of bounds");
+    intf_WarnMsg( 2, "DCT coeff (intra) is out of bounds" );
     p_vpar->picture.b_error = 1;
 }
 
@@ -967,7 +963,7 @@ static __inline__ void DecodeMPEG2NonIntra( vpar_thread_t * p_vpar,
         i_mismatch ^= i_level;
     }
 
-    intf_ErrMsg("vpar error: DCT coeff (non-intra) is out of bounds");
+    intf_WarnMsg( 2, "DCT coeff (non-intra) is out of bounds" );
     p_vpar->picture.b_error = 1;
 }
 
@@ -1118,7 +1114,7 @@ static __inline__ void DecodeMPEG2Intra( vpar_thread_t * p_vpar,
         i_mismatch ^= i_level;
     }
 
-    intf_ErrMsg("vpar error: DCT coeff (intra) is out of bounds");
+    intf_WarnMsg( 2, "DCT coeff (intra) is out of bounds" );
     p_vpar->picture.b_error = 1;
 }
 
@@ -1250,7 +1246,7 @@ static __inline__ int MotionCode( vpar_thread_t * p_vpar )
     if( (i_code -= 12) < 0 )
     {
         p_vpar->picture.b_error = 1;
-        intf_ErrMsg( "vpar error: Invalid motion_vector code" );
+        intf_WarnMsg( 2, "Invalid motion_vector code" );
         return 0;
     }
 
@@ -1634,7 +1630,7 @@ static __inline__ void SkippedMacroblock( vpar_thread_t * p_vpar, int i_mb,
 
     if( i_coding_type == I_CODING_TYPE )
     {
-        intf_ErrMsg("vpar error: skipped macroblock in I-picture");
+        intf_WarnMsg( 2, "skipped macroblock in I-picture" );
         p_vpar->picture.b_error = 1;
         return;
     }
@@ -1770,10 +1766,8 @@ static __inline__ void MacroblockModes( vpar_thread_t * p_vpar,
 #define PARSEERROR                                                      \
 if( p_vpar->picture.b_error )                                           \
 {                                                                       \
-    /* Mark this block as skipped (better than green blocks), and       \
-     * go to the next slice. */                                         \
-    (*pi_mb_address)--;                                                 \
-    vpar_DestroyMacroblock( &p_vpar->vfifo, p_mb );                     \
+    /* Go to the next slice. */                                         \
+    vpar_FreeMacroblock( &p_vpar->vfifo, p_mb );                        \
     return;                                                             \
 }
 
@@ -1799,7 +1793,7 @@ static __inline__ void ParseMacroblock(
 
     if( i_inc < 0 )
     {
-        intf_ErrMsg( "vpar error: bad address increment (%d)", i_inc );
+        intf_WarnMsg( 2, "Bad address increment (%d)", i_inc );
         p_vpar->picture.b_error = 1;
         return;
     }
@@ -1973,10 +1967,8 @@ static __inline__ void ParseMacroblock(
     }
     else
     {
-        /* Mark this block as skipped (better than green blocks), and go
-         * to the next slice. */
-        (*pi_mb_address)--;
-        vpar_DestroyMacroblock( &p_vpar->vfifo, p_mb );
+        /* Go to the next slice. */
+        vpar_FreeMacroblock( &p_vpar->vfifo, p_mb );
     }
 }
 
@@ -1997,8 +1989,6 @@ static __inline__ void SliceHeader( vpar_thread_t * p_vpar,
 {
     int         i_mb_address_save = *pi_mb_address;
 
-    p_vpar->picture.b_error = 0;
-
 #if 0
     /* FIXME : MP@ML doesn't support this. */
     if( b_high )
@@ -2029,7 +2019,7 @@ static __inline__ void SliceHeader( vpar_thread_t * p_vpar,
 
     if( *pi_mb_address < i_mb_address_save )
     {
-        intf_ErrMsg( "vpar error: slices do not follow, maybe a PES has been trashed" );
+        intf_WarnMsg( 2, "Slices do not follow" );
         p_vpar->picture.b_error = 1;
         return;
     }
@@ -2072,13 +2062,14 @@ static __inline__ void vpar_PictureData( vpar_thread_t * p_vpar,
              || !p_vpar->picture.b_error)
            && i_mb_address < (p_vpar->sequence.i_mb_size
                                 >> (i_structure != FRAME_STRUCTURE))
+           && !p_vpar->picture.b_error
            && !p_vpar->p_fifo->b_die )
     {
         if( ((i_dummy = ShowBits( &p_vpar->bit_stream, 32 ))
                  < SLICE_START_CODE_MIN) ||
             (i_dummy > SLICE_START_CODE_MAX) )
         {
-            intf_ErrMsg("vpar error: premature end of picture");
+            intf_WarnMsg( 2, "Premature end of picture" );
             p_vpar->picture.b_error = 1;
             break;
         }
@@ -2089,6 +2080,8 @@ static __inline__ void vpar_PictureData( vpar_thread_t * p_vpar,
                      b_mpeg2, i_coding_type, i_structure );
     }
 
+#if 0
+    /* Buggy */
     /* Try to recover from error. If we missed less than half the
      * number of macroblocks of the picture, mark the missed ones
      * as skipped. */
@@ -2113,6 +2106,7 @@ static __inline__ void vpar_PictureData( vpar_thread_t * p_vpar,
                                i_structure );
         }
     }
+#endif
 }
 
 #define DECLARE_PICD( FUNCNAME, B_MPEG2, I_CODING_TYPE, I_STRUCTURE )       \
@@ -2141,3 +2135,4 @@ DECLARE_PICD( vpar_PictureData2IB, 1, I_CODING_TYPE, BOTTOM_FIELD );
 DECLARE_PICD( vpar_PictureData2PB, 1, P_CODING_TYPE, BOTTOM_FIELD );
 DECLARE_PICD( vpar_PictureData2BB, 1, B_CODING_TYPE, BOTTOM_FIELD );
 #endif
+
